/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.droids.wicket.app;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

import org.apache.droids.exception.DroidsException;
import org.apache.droids.robot.walker.WalkingDroid;
import org.apache.droids.wicket.component.DroidPanel;
import org.apache.wicket.PageParameters;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.spring.injection.annot.SpringBean;


public class IndexPage extends WebPage
{
  @SpringBean( name="droid" )
  WalkingDroid droid;

	/**
	 * Constructor that is invoked when page is invoked without a session.
	 * 
	 * @param parameters
	 *            Page parameters
	 */
	public IndexPage(final PageParameters parameters)
	{
    add( new FeedbackPanel( "feedback" ) );
	  add( new DroidPanel( "droid" ) );
	  
	  final IModel<String> initial = new Model<String>();
	  Form<Void> form = new Form<Void>("form") {
      @Override
      protected void onSubmit() {
        StringTokenizer st = new StringTokenizer( initial.getObject(), "\n\r" );
        if( !st.hasMoreTokens() ) {
          error( "Missing files..." );
        }
        else {
          Collection<File> files = new ArrayList<File>();
          while( st.hasMoreTokens() ) {
            File f = new File( st.nextToken() );
            files.add( f );
            info( "ADD: "+f.getAbsolutePath() );
          }
          droid.setInitialFiles( files );
          try {
            droid.init();
            droid.start();
          }
          catch( DroidsException ex ) {
            throw new RuntimeException("Hey!", ex);
          }
        }
        initial.setObject( null );
      }
	  };
	  form.add( new TextArea<String>("initial", initial ).setRequired( true ) );
	  add( form );
	}
}
