/*
* Licensed to the Apache Software Foundation (ASF) under one
* or more contributor license agreements.  See the NOTICE file
* distributed with this work for additional information
* regarding copyright ownership.  The ASF licenses this file
* to you under the Apache License, Version 2.0 (the
* "License"); you may not use this file except in compliance
* with the License.  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/
package org.apache.droids.wicket.component;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.beanutils.BeanComparator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.ISortState;
import org.apache.wicket.extensions.markup.html.repeater.data.table.ISortableDataProvider;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

public class CollectionDataProvider<T> implements ISortableDataProvider<T> {

  private Collection<T> _values = null;
  private ISortState _sort = null;

  //---------------------------------------------
  //---------------------------------------------

  public CollectionDataProvider( Collection<T> values ) {
    _values = values;
  }
  
  /**
   * @return a thread safe copy of the values as a List
   */
  protected List<T> getValues()
  {
    return new ArrayList<T>( _values );
  }

  @SuppressWarnings("unchecked")
  public IModel<T> model(final T object) {
    if( object instanceof Serializable ) {
      return new Model( (Serializable)object );
    }
    return new AbstractReadOnlyModel<T>() {
      @Override
      public T getObject() {
        return object;
      }
    };
  }
  
  protected String getDefaultSortProperty() {
    return null;
  }
  
  //---------------------------------------------
  //---------------------------------------------

  public void detach() {
    // nothing....
  }

  @Override
  public Iterator<? extends T> iterator(int first, int count) {
    String sortProperty = getDefaultSortProperty();
    boolean sortAsc = true;
    
    // is there any sorting
    if (_sort != null) {
      // ????
    }

    List<T> values = this.getValues();
    
    // perhaps apply a sort...
    if( sortProperty != null ) {
      Comparator<T> comparator = new BeanComparator( sortProperty );
      if( !sortAsc ) {
        comparator = Collections.reverseOrder( comparator );
      }
      Collections.sort( values, comparator );
    }
    
    List<T> sub = values.subList(first, first+count );
    return sub.iterator();
  }

  @Override
  public int size() {
    return getValues().size();
  }

  @Override
  public ISortState getSortState() {
    return _sort;
  }

  @Override
  public void setSortState(ISortState state) {
    _sort = state;
  }
}
