/*
* Licensed to the Apache Software Foundation (ASF) under one
* or more contributor license agreements.  See the NOTICE file
* distributed with this work for additional information
* regarding copyright ownership.  The ASF licenses this file
* to you under the Apache License, Version 2.0 (the
* "License"); you may not use this file except in compliance
* with the License.  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/
package org.apache.droids.wicket.component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.droids.api.Task;
import org.apache.droids.monitor.SimpleWorkMonitor;
import org.apache.droids.monitor.WorkBean;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.ISortState;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;

public class WorkBeanDataProvider<T extends Task> extends CollectionDataProvider<WorkBean<T>> {

  private IModel<SimpleWorkMonitor> monitor;
  private ISortState _sort = null;

  Map<Task, WorkBean<T>> map = null;
  List<WorkBean<T>> list = null;
  
  //---------------------------------------------
  //---------------------------------------------

  public WorkBeanDataProvider( IModel<SimpleWorkMonitor> monitor ) {
    super( null );
    this.monitor = monitor;
  }
  
  
  @SuppressWarnings("unchecked")
  @Override
  public IModel<WorkBean<T>> model( WorkBean<T> object) {
    final Task t = object.getTask();
    return new AbstractReadOnlyModel<WorkBean<T>>() {
      @Override
      public WorkBean<T> getObject() {
        return map.get( t );
      }
    };
  }
  
  @Override
  protected List<WorkBean<T>> getValues()
  {
    // Keep a cache of it so we are not missing the ones that get terminated 
    // between the beginning of the request and the end of the request
    list = new ArrayList<WorkBean<T>>( monitor.getObject().getRunningTasks() );
    map = new HashMap<Task, WorkBean<T>>();
    for( WorkBean<T> bean : list ) {
      map.put( bean.getTask(), bean );
    }
    return list;
  }
  
  //---------------------------------------------
  //---------------------------------------------

  @Override
  public void detach() {
    list = null;
    map = null;
  }
}
